environment

package
v0.19.6 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Mar 26, 2026 License: MIT Imports: 2 Imported by: 0

Documentation

Overview

Package environment for is interacting with environment variables

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func DetectGithub

func DetectGithub() bool

DetectGithub function returns True when the execution environment is detected to be GitHub CI

func FetchEnvVars

func FetchEnvVars(variables []string) map[string]string

FetchEnvVars when provided with list of environment variables is to return a map of variables and values for those that exist in the environment

func LogGroupStart

func LogGroupStart(groupName string)

LogGroupStart is used to group log lines

func LogGroupStop

func LogGroupStop(groupName string)

LogGroupStop is used to group log lines

Types

This section is empty.

Jump to