Documentation
¶
Overview ¶
Package environment for is interacting with environment variables
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
func DetectGithub ¶
func DetectGithub() bool
DetectGithub function returns True when the execution environment is detected to be GitHub CI
func FetchEnvVars ¶
FetchEnvVars when provided with list of environment variables is to return a map of variables and values for those that exist in the environment
Types ¶
This section is empty.
Click to show internal directories.
Click to hide internal directories.